# Generated by Django 3.1.7 on 2021-10-29 16:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0097_auto_20210819_1402'),
    ]

    operations = [
        migrations.AlterField(
            model_name='phase',
            name='estimated_hours',
            field=models.DecimalField(decimal_places=2, default=0.0, max_digits=6),
        ),
    ]



# Generated by Django 2.1.14 on 2020-05-11 11:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0063_accountphysicallocation'),
    ]

    operations = [
        migrations.AddField(
            model_name='ticket',
            name='first_response_date_time',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='ticket',
            name='first_response_due_date_time',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='ticket',
            name='resolution_plan_date_time',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='ticket',
            name='resolution_plan_due_date_time',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='ticket',
            name='resolved_date_time',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='ticket',
            name='resolved_due_date_time',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



from mock import patch
import json
import responses


def create_mock_call(method_name, return_value, side_effect=None):
    """Utility function for mocking the specified function or method"""
    _patch = patch(method_name, side_effect=side_effect)
    mock_get_call = _patch.start()

    if not side_effect:
        mock_get_call.return_value = return_value

    return mock_get_call, _patch


def init_api_rest_connection(return_value=None):
    method_name = 'djautotask.api.get_api_connection_url'
    return create_mock_call(method_name, return_value)


def init_zone_info_connection(return_value=None):
    method_name = 'djautotask.api.get_zone_info'
    return create_mock_call(method_name, return_value)


def service_api_get_ticket_udf_call(return_value):
    method_name = 'djautotask.api.TicketsUDFAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_task_udf_call(return_value):
    method_name = 'djautotask.api.TasksUDFAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_project_udf_call(return_value):
    method_name = 'djautotask.api.ProjectsUDFAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_ticket_call(return_value):
    method_name = 'djautotask.api.TicketsAPIClient.get_single'
    return create_mock_call(method_name, return_value)


def service_api_get_contacts_call(return_value):
    method_name = 'djautotask.api.ContactsAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_contracts_call(return_value):
    method_name = 'djautotask.api.ContractsAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_billing_codes_call(return_value):
    method_name = 'djautotask.api.BillingCodesAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_account_physical_locations_call(return_value):
    method_name = 'djautotask.api.AccountPhysicalLocationsAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_accounts_call(return_value):
    method_name = 'djautotask.api.AccountsAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_roles_call(return_value):
    method_name = 'djautotask.api.RolesAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_departments_call(return_value):
    method_name = 'djautotask.api.DepartmentsAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_resources_call(return_value):
    method_name = 'djautotask.api.ResourcesAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_phases_call(return_value):
    method_name = 'djautotask.api.PhasesAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_note_types_call(return_value):
    method_name = 'djautotask.api.NoteTypesAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_project_note_types_call(return_value):
    method_name = 'djautotask.api.ProjectNoteTypesAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_resource_service_desk_roles_call(return_value):
    method_name = 'djautotask.api.ResourceServiceDeskRolesAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_resource_role_departments_call(return_value):
    method_name = 'djautotask.api.ResourceRoleDepartmentsAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_tickets_call(return_value):
    method_name = 'djautotask.api.TicketsAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_tasks_call(return_value):
    method_name = 'djautotask.api.TasksAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_projects_call(return_value):
    method_name = 'djautotask.api.ProjectsAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_ticket_categories_call(return_value):
    method_name = 'djautotask.api.TicketCategoriesAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_ticket_secondary_resources_call(return_value):
    method_name = 'djautotask.api.TicketSecondaryResourcesAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_task_secondary_resources_call(return_value):
    method_name = 'djautotask.api.TaskSecondaryResourcesAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_ticket_notes_call(return_value):
    method_name = 'djautotask.api.TicketNotesAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_task_notes_call(return_value):
    method_name = 'djautotask.api.TaskNotesAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_time_entries_call(return_value):
    method_name = 'djautotask.api.TimeEntriesAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_task_predecessors_call(return_value):
    method_name = 'djautotask.api.TaskPredecessorsAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_ticket_category_picklist_call(return_value):
    method_name = 'djautotask.api.TicketCategoryPicklistAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_ticket_picklist_call(return_value):
    method_name = 'djautotask.api.TicketPicklistAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_task_picklist_call(return_value):
    method_name = 'djautotask.api.TaskPicklistAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_project_picklist_call(return_value):
    method_name = 'djautotask.api.ProjectPicklistAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_license_types_call(return_value):
    method_name = 'djautotask.api.LicenseTypesAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_service_call_statuses_call(return_value):
    method_name = 'djautotask.api.ServiceCallStatusPicklistAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_service_calls_call(return_value):
    method_name = 'djautotask.api.ServiceCallsAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_service_call_tickets_call(return_value):
    method_name = 'djautotask.api.ServiceCallTicketsAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_service_call_ticket_resources_call(return_value):
    method_name = 'djautotask.api.ServiceCallTicketResourcesAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_service_call_tasks_call(return_value):
    method_name = 'djautotask.api.ServiceCallTasksAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_service_call_task_resources_call(return_value):
    method_name = 'djautotask.api.ServiceCallTaskResourcesAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_use_types_call(return_value):
    method_name = 'djautotask.api.UseTypesAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_billing_code_types_call(return_value):
    method_name = 'djautotask.api.BillingCodeTypeAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_task_type_links_call(return_value):
    method_name = \
        'djautotask.api.TaskTypeLinksAPIClient.get'
    return create_mock_call(method_name, return_value)


def service_api_get_account_types_call(return_value):
    method_name = 'djautotask.api.AccountTypesAPIClient.get'
    return create_mock_call(method_name, return_value)


def get(url, data, headers=None, status=200):
    """Set up requests mock for given URL and JSON-serializable data."""
    get_raw(url, json.dumps(data), "application/json", headers, status=status)


def get_raw(url, data, content_type="application/octet-stream", headers=None,
            status=200):
    """Set up requests mock for given URL."""
    responses.add(
        responses.GET,
        url,
        body=data,
        status=status,
        content_type=content_type,
        adding_headers=headers,
    )



# Generated by Django 2.1.11 on 2019-10-21 10:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0016_auto_20191021_0958'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='displaycolor',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='issuetype',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='projectstatus',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='projecttype',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='queue',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='source',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='subissuetype',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='ticketpriority',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='ticketstatus',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='tickettype',
            managers=[
            ],
        ),
    ]



# Generated by Django 2.1.11 on 2019-09-20 16:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0007_auto_20190917_1009'),
        ('djautotask', '0006_merge_20190920_1450'),
    ]

    operations = [
    ]



import pytz
from unittest.mock import patch

from django.test import TestCase
from django.utils import timezone
from django.test import override_settings
from datetime import timedelta

from djautotask.models import TimeEntry, OFFSET_TIMEZONE, Ticket
from . import mocks as mk


class TestTicket(TestCase):
    API_URL = 'https://localhost/'

    def setUp(self):
        mk.init_api_rest_connection(return_value=self.API_URL)

    def test_save_calls_update_at_when_kwarg_passed(self):
        ticket = Ticket.objects.create(
            title='test',
            due_date_time=timezone.now(),
        )
        with patch('djautotask.api.TicketsAPIClient') as mock_ticketapiclient:
            instance = mock_ticketapiclient.return_value
            ticket.save()
            self.assertFalse(instance.legacy_update.called)
            ticket.save(update_at=True)
            self.assertFalse(instance.legacy_update.called)
            # 'update_at' is called
            ticket.save(update_at=True,
                        changed_fields=['title', 'due_date_time'])
            self.assertTrue(instance.legacy_update.called)


class TestTimeEntry(TestCase):

    def test_get_entered_time_has_end_date_time(self):
        """
        Test that get_entered_time returns a time entry's end_date_time.
        """
        end_time = timezone.now() + timedelta(hours=1)
        time_entry = TimeEntry()
        time_entry.start_date_time = timezone.now()
        time_entry.end_date_time = end_time

        self.assertEqual(time_entry.get_entered_time(), end_time)

    def assert_get_entered_time_date_worked(self, local_midnight):
        """
        Verify that the time of date_worked is set to midnight local time.
        Autotask gives us date_worked's time as EST 00:00:00 which then
        gets converted to UTC 05:00:00.
        """
        midnight_est = timezone.localtime(
            timezone=pytz.timezone(OFFSET_TIMEZONE)
        ).replace(
            year=local_midnight.year,
            month=local_midnight.month, day=local_midnight.day,
            hour=0, minute=0, second=0, microsecond=0
        )
        # Convert to UTC just like Django does when we receive Autotask data.
        date_worked = midnight_est.astimezone(pytz.utc)
        time_entry = TimeEntry()
        time_entry.date_worked = date_worked

        # Convert our local datetime to UTC since that is
        # what get_entered_time returns.
        local_midnight_utc = local_midnight.astimezone(pytz.utc)
        self.assertEqual(time_entry.get_entered_time(), local_midnight_utc)

    @override_settings(TIME_ZONE='America/Vancouver')
    def test_get_entered_time_date_worked_pst(self):

        local_midnight = timezone.localtime().replace(
            hour=0, minute=0, second=0, microsecond=0)

        self.assert_get_entered_time_date_worked(local_midnight)

    @override_settings(TIME_ZONE='America/New_York')
    def test_get_entered_time_date_worked_est(self):

        local_midnight = timezone.localtime().replace(
            hour=0, minute=0, second=0, microsecond=0)

        self.assert_get_entered_time_date_worked(local_midnight)

    # This is a test for specific time, new year eve
    # @freeze_time("2020-12-31 15:00:00")
    @override_settings(TIME_ZONE='Australia/Sydney')
    def test_get_entered_time_date_worked_australia(self):

        local_midnight = timezone.localtime().replace(
            hour=0, minute=0, second=0, microsecond=0)

        self.assert_get_entered_time_date_worked(local_midnight)

    @override_settings(TIME_ZONE='Europe/Paris')
    def test_get_entered_time_date_worked_paris(self):

        local_midnight = timezone.localtime().replace(
            hour=0, minute=0, second=0, microsecond=0)

        self.assert_get_entered_time_date_worked(local_midnight)



# Generated by Django 2.1.11 on 2019-10-31 16:20

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0021_merge_20191031_0858'),
        ('djautotask', '0021_merge_20191031_1259'),
    ]

    operations = [
    ]



# Generated by Django 3.1.5 on 2021-01-25 16:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0088_merge_20210113_1743'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contact',
            name='alternate_phone',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name='contact',
            name='email_address',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name='contact',
            name='email_address2',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name='contact',
            name='email_address3',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name='contact',
            name='first_name',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name='contact',
            name='last_name',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name='contact',
            name='mobile_phone',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name='contact',
            name='phone',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
    ]



# Generated by Django 2.1.14 on 2019-11-19 09:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0022_merge_20191031_1620'),
    ]

    operations = [
        migrations.AlterField(
            model_name='project',
            name='duration',
            field=models.PositiveIntegerField(default=0),
        ),
    ]



import io
from django.core.management import call_command
from django.test import TestCase
from djautotask.tests import fixtures, mocks, fixture_utils
from djautotask import models


def sync_summary(class_name, created_count, updated_count=0):
    return '{} Sync Summary - Created: {}, Updated: {}, Skipped: 0'.format(
        class_name, created_count, updated_count
    )


def full_sync_summary(class_name, deleted_count, updated_count=0):
    return '{} Sync Summary - Created: 0, Updated: {}, Skipped: 0, ' \
        'Deleted: {}'.format(class_name, updated_count, deleted_count)


def slug_to_title(slug):
    return slug.title().replace('_', ' ')


def run_sync_command(full_option=False, command_name=None):
    out = io.StringIO()
    args = ['atsync']

    if command_name:
        args.append(command_name)

    if full_option:
        args.append('--full')

    call_command(*args, stdout=out)

    return out


class AbstractBaseSyncTest(object):

    def setUp(self):
        super().setUp()
        mocks.init_api_rest_connection()

    def _test_sync(self, mock_call, return_value, at_object,
                   full_option=False):
        mock_call(return_value)
        out = io.StringIO()

        args = ['atsync', at_object]
        if full_option:
            args.append('--full')
        call_command(*args, stdout=out)
        return out

    def _title_for_at_object(self, at_object):
        return at_object.title().replace('_', ' ')

    def test_sync(self):
        out = self._test_sync(*self.args)
        obj_title = self._title_for_at_object(self.args[-1])
        self.assertIn(obj_title, out.getvalue().strip())

    def test_full_sync(self):
        self.test_sync()
        mock_call, return_value, at_object = self.args
        args = [
            mock_call,
            {
                "items": [],
                "pageDetails": fixtures.API_PAGE_DETAILS
            },
            at_object
        ]

        out = self._test_sync(*args, full_option=True)
        obj_label = self._title_for_at_object(at_object)
        msg_tmpl = '{} Sync Summary - Created: 0, Updated: 0, Skipped: 0, ' \
                   'Deleted: {}'
        msg = msg_tmpl.format(obj_label, len(return_value.get('items')))
        self.assertEqual(msg, out.getvalue().strip())


class PicklistSyncTest(AbstractBaseSyncTest):

    def test_full_sync(self):
        self.test_sync()
        mock_call, return_value, at_object = self.args
        args = [
            mock_call,
            {
                "fields": []
            },
            at_object
        ]

        out = self._test_sync(*args, full_option=True)
        obj_label = self._title_for_at_object(at_object)
        msg_tmpl = '{} Sync Summary - Created: 0, Updated: 0, Skipped: 0, ' \
                   'Deleted: {}'
        msg = msg_tmpl.format(
            obj_label, len(return_value.get('fields')[0].get('picklistValues'))
        )
        self.assertEqual(msg, out.getvalue().strip())


class TestSyncTicketUDFCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_ticket_udf_call,
        fixtures.API_UDF,
        'ticket_udf',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_ticket_udfs()


class TestSyncTaskUDFCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_task_udf_call,
        fixtures.API_UDF,
        'task_udf',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_task_udfs()


class TestSyncProjectUDFCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_project_udf_call,
        fixtures.API_UDF,
        'project_udf',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_project_udfs()


class TestSyncContactCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_contacts_call,
        fixtures.API_CONTACT,
        'contact',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_contacts()


class TestSyncTicketCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_tickets_call,
        fixtures.API_TICKET,
        'ticket',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_tickets()


class TestSyncStatusCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_ticket_picklist_call,
        fixtures.API_STATUS_FIELD,
        'status',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_statuses()


class TestSyncPriorityCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_ticket_picklist_call,
        fixtures.API_PRIORITY_FIELD,
        'priority',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_priorities()


class TestSyncQueueCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_ticket_picklist_call,
        fixtures.API_QUEUE_FIELD,
        'queue',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_queues()


class TestSyncProjectStatusCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_project_picklist_call,
        fixtures.API_PROJECT_STATUS_FIELD,
        'project_status',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_project_statuses()


class TestSyncProjectTypeCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_project_picklist_call,
        fixtures.API_PROJECT_TYPE_FIELD,
        'project_type',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_project_types()


class TestSyncSourceCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_ticket_picklist_call,
        fixtures.API_SOURCE_FIELD,
        'source',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_sources()


class TestSyncIssueTypeCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_ticket_picklist_call,
        fixtures.API_ISSUE_TYPE_FIELD,
        'issue_type',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_issue_types()


class TestSyncSubIssueTypeCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_ticket_picklist_call,
        fixtures.API_SUB_ISSUE_TYPE_FIELD,
        'sub_issue_type',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_issue_types()
        fixture_utils.init_sub_issue_types()


class TestSyncTicketTypeCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_ticket_picklist_call,
        fixtures.API_TICKET_TYPE_FIELD,
        'ticket_type',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_ticket_types()


class TestSyncAccountTypeCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_account_types_call,
        fixtures.API_ACCOUNT_TYPE_FIELD,
        'account_type',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_account_types()


class TestSyncServiceCallStatusCommand(PicklistSyncTest,
                                       TestCase):
    args = (
        mocks.service_api_get_service_call_statuses_call,
        fixtures.API_SERVICE_CALL_STATUS_FIELD,
        'service_call_status',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_service_call_statuses()


class TestSyncDisplayColorCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_ticket_category_picklist_call,
        fixtures.API_DISPLAY_COLOR_FIELD,
        'display_color',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_display_colors()


class TestSyncNoteTypeCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_note_types_call,
        fixtures.API_NOTE_TYPE_FIELD,
        'note_type',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_note_types()


class TestProjectNoteTypeCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_project_note_types_call,
        fixtures.API_PROJECT_NOTE_TYPE_FIELD,
        'project_note_type',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_project_note_types()


class TestSyncLicenseTypeCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_license_types_call,
        fixtures.API_LICENSE_TYPE_FIELD,
        'license_type',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_license_types()


class TestSyncTaskTypeLinkCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_task_type_links_call,
        fixtures.API_TASK_TYPE_LINK_FIELD,
        'task_type_link',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_task_type_links()


class TestSyncTaskCategoryCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_task_picklist_call,
        fixtures.API_TASK_CATEGORY_FIELD,
        'task_category',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_task_categories()


class TestSyncUseTypeCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_use_types_call,
        fixtures.API_USE_TYPE_FIELD,
        'use_type',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_use_types()


class TestSyncBillingCodeTypeCommand(PicklistSyncTest, TestCase):
    args = (
        mocks.service_api_get_billing_code_types_call,
        fixtures.API_BILLING_CODE_TYPE_FIELD,
        'billing_code_type',
    )


class TestSyncTicketCategoryCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_ticket_categories_call,
        fixtures.API_TICKET_CATEGORY,
        'ticket_category',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_ticket_categories()


class TestSyncResourceCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_resources_call,
        fixtures.API_RESOURCE,
        'resource',
    )


class TestSyncAccountCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_accounts_call,
        fixtures.API_ACCOUNT,
        'account',
    )


class TestSyncAccountLocationCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_account_physical_locations_call,
        fixtures.API_ACCOUNT_PHYSICAL_LOCATION,
        'account_physical_location',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_accounts()
        fixture_utils.init_account_physical_locations()


class TestSyncProjectCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_projects_call,
        fixtures.API_PROJECT,
        'project',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_projects()


class TestSyncPhaseCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_phases_call,
        fixtures.API_PHASE,
        'phase',
    )


class TestSyncTaskCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_tasks_call,
        fixtures.API_TASK,
        'task',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_projects()
        fixture_utils.init_tasks()


class TestSyncTicketSecondaryResourceCommand(AbstractBaseSyncTest,
                                             TestCase):
    args = (
        mocks.service_api_get_ticket_secondary_resources_call,
        fixtures.API_TICKET_SECONDARY_RESOURCE,
        'ticket_secondary_resource',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_tickets()


class TestSyncTaskSecondaryResourceCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_task_secondary_resources_call,
        fixtures.API_TASK_SECONDARY_RESOURCE,
        'task_secondary_resource',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_projects()
        fixture_utils.init_tasks()


class TestSyncTicketNoteCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_ticket_notes_call,
        fixtures.API_TICKET_NOTE,
        'ticket_note',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_tickets()
        fixture_utils.init_ticket_notes()


class TestSyncTaskNoteCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_task_notes_call,
        fixtures.API_TASK_NOTE,
        'task_note',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_projects()
        fixture_utils.init_tasks()
        fixture_utils.init_task_notes()


class TestSyncTimeEntryCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_time_entries_call,
        fixtures.API_TIME_ENTRY,
        'time_entry',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_resources()
        fixture_utils.init_projects()
        fixture_utils.init_tasks()
        fixture_utils.init_tickets()
        fixture_utils.init_time_entries()


class TestSyncBillingCodeCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_billing_codes_call,
        fixtures.API_BILLING_CODE,
        'billing_code',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_billing_codes()


class TestSyncRoleCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_roles_call,
        fixtures.API_ROLE,
        'role',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_roles()


class TestSyncDepartmentCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_departments_call,
        fixtures.API_DEPARTMENT,
        'department',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_departments()


class TestSyncResourceServiceDeskRoleCommand(AbstractBaseSyncTest,
                                             TestCase):
    args = (
        mocks.service_api_get_resource_service_desk_roles_call,
        fixtures.API_RESOURCE_SERVICE_DESK_ROLE,
        'resource_service_desk_role',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_roles()
        fixture_utils.init_resources()
        fixture_utils.init_resource_service_desk_roles()


class TestSyncResourceRoleDepartmentCommand(AbstractBaseSyncTest,
                                            TestCase):
    args = (
        mocks.service_api_get_resource_role_departments_call,
        fixtures.API_RESOURCE_ROLE_DEPARTMENT,
        'resource_role_department',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_departments()
        fixture_utils.init_roles()
        fixture_utils.init_resources()
        fixture_utils.init_resource_role_departments()


class TestSyncContractCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_contracts_call,
        fixtures.API_CONTRACT,
        'contract',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_contracts()


class TestSyncServiceCallCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_service_calls_call,
        fixtures.API_SERVICE_CALL,
        'service_call',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_service_call_statuses()
        fixture_utils.init_resources()
        fixture_utils.init_account_types()
        fixture_utils.init_accounts()


class TestSyncServiceCallTicketCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_service_call_tickets_call,
        fixtures.API_SERVICE_CALL_TICKET,
        'service_call_ticket',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_service_call_statuses()
        fixture_utils.init_resources()
        fixture_utils.init_account_types()
        fixture_utils.init_accounts()
        fixture_utils.init_service_calls()
        fixture_utils.init_statuses()
        fixture_utils.init_tickets()


class TestSyncServiceCallTaskCommand(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_service_call_tasks_call,
        fixtures.API_SERVICE_CALL_TASK,
        'service_call_task',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_service_call_statuses()
        fixture_utils.init_account_types()
        fixture_utils.init_accounts()
        fixture_utils.init_service_calls()
        fixture_utils.init_statuses()
        fixture_utils.init_projects()
        fixture_utils.init_tasks()


class TestSyncServiceCallTicketResourceCommand(AbstractBaseSyncTest,
                                               TestCase):
    args = (
        mocks.service_api_get_service_call_ticket_resources_call,
        fixtures.API_SERVICE_CALL_TICKET_RESOURCE,
        'service_call_ticket_resource',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_service_call_statuses()
        fixture_utils.init_resources()
        fixture_utils.init_account_types()
        fixture_utils.init_accounts()
        fixture_utils.init_service_calls()
        fixture_utils.init_statuses()
        fixture_utils.init_tickets()
        fixture_utils.init_service_call_tickets()


class TestSyncServiceCallTaskResourceCommand(AbstractBaseSyncTest,
                                             TestCase):
    args = (
        mocks.service_api_get_service_call_task_resources_call,
        fixtures.API_SERVICE_CALL_TASK_RESOURCE,
        'service_call_task_resource',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_service_call_statuses()
        fixture_utils.init_resources()
        fixture_utils.init_account_types()
        fixture_utils.init_accounts()
        fixture_utils.init_service_calls()
        fixture_utils.init_statuses()
        fixture_utils.init_projects()
        fixture_utils.init_tasks()
        fixture_utils.init_service_call_tasks()


class TestSyncTaskPredecessor(AbstractBaseSyncTest, TestCase):
    args = (
        mocks.service_api_get_task_predecessors_call,
        fixtures.API_TASK_PREDECESSOR,
        'task_predecessor',
    )

    def setUp(self):
        super().setUp()
        fixture_utils.init_projects()
        fixture_utils.init_tasks()
        fixture_utils.init_task_predecessors()


class TestSyncAllCommand(TestCase):

    def setUp(self):
        super().setUp()
        mocks.init_api_rest_connection()
        self._call_service_api()

        sync_test_cases = [
            TestSyncTicketUDFCommand,
            TestSyncTaskUDFCommand,
            TestSyncProjectUDFCommand,
            TestSyncNoteTypeCommand,
            TestProjectNoteTypeCommand,
            TestSyncLicenseTypeCommand,
            TestSyncTaskTypeLinkCommand,
            TestSyncUseTypeCommand,
            TestSyncBillingCodeTypeCommand,
            TestSyncAccountTypeCommand,
            TestSyncRoleCommand,
            TestSyncDepartmentCommand,
            TestSyncTicketCommand,
            TestSyncTaskCategoryCommand,
            TestSyncTaskCommand,
            TestSyncStatusCommand,
            TestSyncResourceCommand,
            TestSyncPriorityCommand,
            TestSyncQueueCommand,
            TestSyncAccountCommand,
            TestSyncProjectCommand,
            TestSyncProjectStatusCommand,
            TestSyncProjectTypeCommand,
            TestSyncTicketCategoryCommand,
            TestSyncSourceCommand,
            TestSyncIssueTypeCommand,
            TestSyncSubIssueTypeCommand,
            TestSyncTicketTypeCommand,
            TestSyncDisplayColorCommand,
            TestSyncTaskSecondaryResourceCommand,
            TestSyncPhaseCommand,
            TestSyncTicketNoteCommand,
            TestSyncTaskNoteCommand,
            TestSyncTimeEntryCommand,
            TestSyncBillingCodeCommand,
            TestSyncResourceRoleDepartmentCommand,
            TestSyncResourceServiceDeskRoleCommand,
            TestSyncContractCommand,
            TestSyncServiceCallStatusCommand,
            TestSyncServiceCallCommand,
            TestSyncServiceCallTicketCommand,
            TestSyncServiceCallTaskCommand,
            TestSyncServiceCallTicketResourceCommand,
            TestSyncServiceCallTaskResourceCommand,
            TestSyncAccountLocationCommand,
            TestSyncTaskPredecessor,
            TestSyncContactCommand,
        ]
        self.test_args = []

        for test_case in sync_test_cases:
            self.test_args.append(test_case.args)

    def test_partial_sync(self):
        """
        Test the command to run a sync of all objects without
        the --full argument.
        """
        output = run_sync_command()

        for mock_call, fixture, at_object in self.test_args:
            if mock_call:
                if 'fields' in fixture:
                    fixture_len = \
                        len(fixture.get('fields')[0].get('picklistValues'))
                else:
                    fixture_len = len(fixture.get('items'))
            else:
                fixture_len = len(fixture)
            summary = sync_summary(slug_to_title(at_object), fixture_len)
            self.assertIn(summary, output.getvalue().strip())

        self.assertEqual(
            models.Ticket.objects.all().count(),
            len(fixtures.API_TICKET['items'])
        )

    def test_full_sync(self):
        """Test the command to run a full sync of all objects."""
        at_object_map = {
            'ticket_udf': models.TicketUDF,
            'task_udf': models.TaskUDF,
            'project_udf': models.ProjectUDF,
            'note_type': models.NoteType,
            'account_type': models.AccountType,
            'role': models.Role,
            'department': models.Department,
            'status': models.Status,
            'priority': models.Priority,
            'queue': models.Queue,
            'source': models.Source,
            'issue_type': models.IssueType,
            'display_color': models.DisplayColor,
            'ticket': models.Ticket,
            'resource': models.Resource,
            'ticket_secondary_resource': models.TicketSecondaryResource,
            'account': models.Account,
            'account_physical_location': models.AccountPhysicalLocation,
            'project': models.Project,
            'project_status': models.ProjectStatus,
            'project_type': models.ProjectType,
            'project_note_type': models.ProjectNoteType,
            'ticket_category': models.TicketCategory,
            'sub_issue_type': models.SubIssueType,
            'ticket_type': models.TicketType,
            'license_type': models.LicenseType,
            'task_category': models.TaskCategory,
            'task': models.Task,
            'task_secondary_resource': models.TaskSecondaryResource,
            'phase': models.Phase,
            'ticket_note': models.TicketNote,
            'task_note': models.TaskNote,
            'time_entry': models.TimeEntry,
            'task_type_link': models.TaskTypeLink,
            'use_type': models.UseType,
            'billing_code_type': models.BillingCodeType,
            'billing_code': models.BillingCode,
            'resource_role_department': models.ResourceRoleDepartment,
            'resource_service_desk_role': models.ResourceServiceDeskRole,
            'contract': models.Contract,
            'service_call_status': models.ServiceCallStatus,
            'service_call': models.ServiceCall,
            'service_call_ticket': models.ServiceCallTicket,
            'service_call_task': models.ServiceCallTask,
            'service_call_ticket_resource': models.ServiceCallTicketResource,
            'service_call_task_resource': models.ServiceCallTaskResource,
            'task_predecessor': models.TaskPredecessor,
            'contact': models.Contact,
        }
        run_sync_command()
        pre_full_sync_counts = {}

        self._call_empty_service_api()
        for key, model_class in at_object_map.items():
            pre_full_sync_counts[key] = model_class.objects.all().count()

        output = run_sync_command(full_option=True)

        # Verify the rest of sync classes summaries.
        for mock_call, fixture, at_object in self.test_args:
            if at_object in (
                    'resource_role_department',
                    'resource_service_desk_role',
                    'service_call',
                    'service_call_ticket',
                    'service_call_task',
                    'service_call_ticket_resource',
                    'service_call_task_resource',
                    'task_predecessor',
                    'task',
                    'time_entry'
            ):
                # Assert that there were objects to get deleted, then change
                # to zero to verify the output formats correctly.
                # We are just testing the command, there are sync tests to
                # verify that the synchronizers work correctly
                self.assertGreater(pre_full_sync_counts[at_object], 0)
                pre_full_sync_counts[at_object] = 0
            summary = full_sync_summary(
                slug_to_title(at_object),
                pre_full_sync_counts[at_object]
            )
            self.assertIn(summary, output.getvalue().strip())

    def _call_service_api(self):
        mocks.service_api_get_ticket_udf_call(fixtures.API_UDF)
        mocks.service_api_get_task_udf_call(fixtures.API_UDF)
        mocks.service_api_get_project_udf_call(fixtures.API_UDF)
        mocks.service_api_get_roles_call(fixtures.API_ROLE)
        mocks.service_api_get_departments_call(fixtures.API_DEPARTMENT)
        mocks.service_api_get_resource_service_desk_roles_call(
            fixtures.API_RESOURCE_SERVICE_DESK_ROLE)
        mocks.service_api_get_resource_role_departments_call(
            fixtures.API_RESOURCE_ROLE_DEPARTMENT)
        mocks.service_api_get_license_types_call(
            fixtures.API_LICENSE_TYPE_FIELD)
        mocks.service_api_get_use_types_call(fixtures.API_USE_TYPE_FIELD)
        mocks.service_api_get_billing_code_types_call(
            fixtures.API_BILLING_CODE_TYPE_FIELD)
        mocks.service_api_get_task_type_links_call(
            fixtures.API_TASK_TYPE_LINK_FIELD)
        mocks.service_api_get_account_types_call(
            fixtures.API_ACCOUNT_TYPE_FIELD)
        mocks.service_api_get_ticket_category_picklist_call(
            fixtures.API_DISPLAY_COLOR_FIELD)
        mocks.service_api_get_ticket_picklist_call(
            fixtures.API_TICKET_PICKLIST_FIELD)
        mocks.service_api_get_project_picklist_call(
            fixtures.API_PROJECT_PICKLIST_FIELD)
        mocks.service_api_get_service_call_statuses_call(
            fixtures.API_SERVICE_CALL_STATUS_FIELD)
        mocks.service_api_get_note_types_call(fixtures.API_NOTE_TYPE_FIELD)
        mocks.service_api_get_project_note_types_call(
            fixtures.API_PROJECT_NOTE_TYPE_FIELD)
        mocks.service_api_get_task_picklist_call(
            fixtures.API_TASK_CATEGORY_FIELD)

        mocks.service_api_get_contacts_call(fixtures.API_CONTACT)
        mocks.service_api_get_contracts_call(fixtures.API_CONTRACT)
        mocks.service_api_get_billing_codes_call(
            fixtures.API_BILLING_CODE)
        mocks.service_api_get_account_physical_locations_call(
            fixtures.API_ACCOUNT_PHYSICAL_LOCATION)
        mocks.service_api_get_ticket_categories_call(
            fixtures.API_TICKET_CATEGORY)
        mocks.service_api_get_tickets_call(fixtures.API_TICKET)
        mocks.service_api_get_tasks_call(fixtures.API_TASK)
        mocks.service_api_get_projects_call(fixtures.API_PROJECT)
        mocks.service_api_get_service_calls_call(fixtures.API_SERVICE_CALL)
        mocks.service_api_get_service_call_tickets_call(
            fixtures.API_SERVICE_CALL_TICKET)
        mocks.service_api_get_resources_call(fixtures.API_RESOURCE)
        mocks.service_api_get_accounts_call(fixtures.API_ACCOUNT)
        mocks.service_api_get_phases_call(fixtures.API_PHASE)
        mocks.service_api_get_service_call_ticket_resources_call(
            fixtures.API_SERVICE_CALL_TICKET_RESOURCE)
        mocks.service_api_get_service_call_tasks_call(
            fixtures.API_SERVICE_CALL_TASK)
        mocks.service_api_get_ticket_secondary_resources_call(
            fixtures.API_TICKET_SECONDARY_RESOURCE)
        mocks.service_api_get_task_secondary_resources_call(
            fixtures.API_TASK_SECONDARY_RESOURCE)
        mocks.service_api_get_ticket_notes_call(fixtures.API_TICKET_NOTE)
        mocks.service_api_get_task_notes_call(fixtures.API_TASK_NOTE)
        mocks.service_api_get_time_entries_call(fixtures.API_TIME_ENTRY)
        mocks.service_api_get_service_call_task_resources_call(
            fixtures.API_SERVICE_CALL_TASK_RESOURCE)
        mocks.service_api_get_task_predecessors_call(
            fixtures.API_TASK_PREDECESSOR)

    def _call_empty_service_api(self):
        mocks.service_api_get_ticket_udf_call(fixtures.API_EMPTY_FIELDS)
        mocks.service_api_get_task_udf_call(fixtures.API_EMPTY_FIELDS)
        mocks.service_api_get_project_udf_call(fixtures.API_EMPTY_FIELDS)
        mocks.service_api_get_contacts_call(fixtures.API_EMPTY)
        mocks.service_api_get_contracts_call(fixtures.API_EMPTY)
        mocks.service_api_get_billing_codes_call(fixtures.API_EMPTY)
        mocks.service_api_get_account_physical_locations_call(
            fixtures.API_EMPTY)
        mocks.service_api_get_tickets_call(fixtures.API_EMPTY)
        mocks.service_api_get_tasks_call(fixtures.API_EMPTY)
        mocks.service_api_get_projects_call(fixtures.API_EMPTY)
        mocks.service_api_get_ticket_categories_call(fixtures.API_EMPTY)
        mocks.service_api_get_task_predecessors_call(fixtures.API_EMPTY)
        mocks.service_api_get_roles_call(fixtures.API_EMPTY)
        mocks.service_api_get_departments_call(fixtures.API_EMPTY)
        mocks.service_api_get_resource_service_desk_roles_call(
            fixtures.API_EMPTY)
        mocks.service_api_get_resource_role_departments_call(
            fixtures.API_EMPTY)
        mocks.service_api_get_service_calls_call(fixtures.API_EMPTY)
        mocks.service_api_get_service_call_tickets_call(fixtures.API_EMPTY)
        mocks.service_api_get_resources_call(fixtures.API_EMPTY)
        mocks.service_api_get_accounts_call(fixtures.API_EMPTY)
        mocks.service_api_get_phases_call(fixtures.API_EMPTY)
        mocks.service_api_get_service_call_ticket_resources_call(
            fixtures.API_EMPTY)
        mocks.service_api_get_service_call_tasks_call(fixtures.API_EMPTY)
        mocks.service_api_get_ticket_secondary_resources_call(
            fixtures.API_EMPTY)
        mocks.service_api_get_task_secondary_resources_call(
            fixtures.API_EMPTY)
        mocks.service_api_get_ticket_notes_call(fixtures.API_EMPTY)
        mocks.service_api_get_task_notes_call(fixtures.API_EMPTY)
        mocks.service_api_get_time_entries_call(fixtures.API_EMPTY)
        mocks.service_api_get_service_call_task_resources_call(
            fixtures.API_EMPTY)
        mocks.service_api_get_ticket_category_picklist_call(
            fixtures.API_EMPTY_FIELDS)
        mocks.service_api_get_ticket_picklist_call(fixtures.API_EMPTY_FIELDS)
        mocks.service_api_get_project_picklist_call(fixtures.API_EMPTY_FIELDS)
        mocks.service_api_get_license_types_call(fixtures.API_EMPTY_FIELDS)
        mocks.service_api_get_use_types_call(fixtures.API_EMPTY_FIELDS)
        mocks.service_api_get_billing_code_types_call(
            fixtures.API_EMPTY_FIELDS)
        mocks.service_api_get_task_type_links_call(fixtures.API_EMPTY_FIELDS)
        mocks.service_api_get_account_types_call(fixtures.API_EMPTY_FIELDS)
        mocks.service_api_get_service_call_statuses_call(
            fixtures.API_EMPTY_FIELDS)
        mocks.service_api_get_note_types_call(fixtures.API_EMPTY_FIELDS)
        mocks.service_api_get_project_note_types_call(
            fixtures.API_EMPTY_FIELDS)
        mocks.service_api_get_task_picklist_call(fixtures.API_EMPTY_FIELDS)



# Generated by Django 2.1.14 on 2020-02-04 15:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0040_merge_20200131_1627'),
        ('djautotask', '0040_auto_20200204_1538'),
    ]

    operations = [
    ]



# Generated by Django 4.0.7 on 2022-10-14 15:56

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0108_billingcodetype_billingcodetypetracker'),
    ]

    operations = [
        migrations.AddField(
            model_name='billingcode',
            name='billing_code_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djautotask.billingcodetype'),
        ),
    ]



# Generated by Django 3.1.2 on 2020-10-09 10:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0079_auto_20201003_1125'),
    ]

    operations = [
        migrations.CreateModel(
            name='AccountPhysicalLocationTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_accountphysicallocation',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.accountphysicallocation',),
        ),
        migrations.CreateModel(
            name='AccountTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_account',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.account',),
        ),
        migrations.CreateModel(
            name='AccountTypeTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_accounttype',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.accounttype',),
        ),
        migrations.CreateModel(
            name='AllocationCodeTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_allocationcode',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.allocationcode',),
        ),
        migrations.CreateModel(
            name='ContractTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_contract',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.contract',),
        ),
        migrations.CreateModel(
            name='DepartmentTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_department',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.department',),
        ),
        migrations.CreateModel(
            name='DisplayColorTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_displaycolor',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.displaycolor',),
        ),
        migrations.CreateModel(
            name='IssueTypeTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_issuetype',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.issuetype',),
        ),
        migrations.CreateModel(
            name='LicenseTypeTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_licensetype',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.licensetype',),
        ),
        migrations.CreateModel(
            name='NoteTypeTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_notetype',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.notetype',),
        ),
        migrations.CreateModel(
            name='PhaseTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_phase',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.phase',),
        ),
        migrations.CreateModel(
            name='PriorityTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_priority',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.priority',),
        ),
        migrations.CreateModel(
            name='ProjectStatusTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_projectstatus',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.projectstatus',),
        ),
        migrations.CreateModel(
            name='ProjectTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_project',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.project',),
        ),
        migrations.CreateModel(
            name='ProjectTypeTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_projecttype',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.projecttype',),
        ),
        migrations.CreateModel(
            name='QueueTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_queue',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.queue',),
        ),
        migrations.CreateModel(
            name='ResourceRoleDepartmentTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_resourceroledepartment',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.resourceroledepartment',),
        ),
        migrations.CreateModel(
            name='ResourceServiceDeskRoleTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_resourceservicedeskrole',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.resourceservicedeskrole',),
        ),
        migrations.CreateModel(
            name='ResourceTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_resource',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.resource',),
        ),
        migrations.CreateModel(
            name='RoleTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_role',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.role',),
        ),
        migrations.CreateModel(
            name='ServiceCallStatusTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_servicecallstatus',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.servicecallstatus',),
        ),
        migrations.CreateModel(
            name='ServiceCallTaskResourceTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_servicecalltaskresource',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.servicecalltaskresource',),
        ),
        migrations.CreateModel(
            name='ServiceCallTaskTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_servicecalltask',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.servicecalltask',),
        ),
        migrations.CreateModel(
            name='ServiceCallTicketResourceTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_servicecallticketresource',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.servicecallticketresource',),
        ),
        migrations.CreateModel(
            name='ServiceCallTicketTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_servicecallticket',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.servicecallticket',),
        ),
        migrations.CreateModel(
            name='ServiceCallTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_servicecall',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.servicecall',),
        ),
        migrations.CreateModel(
            name='SourceTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_source',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.source',),
        ),
        migrations.CreateModel(
            name='StatusTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_status',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.status',),
        ),
        migrations.CreateModel(
            name='SubIssueTypeTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_subissuetype',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.subissuetype',),
        ),
        migrations.CreateModel(
            name='TaskNoteTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_tasknote',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.tasknote',),
        ),
        migrations.CreateModel(
            name='TaskPredecessorTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_taskpredecessor',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.taskpredecessor',),
        ),
        migrations.CreateModel(
            name='TaskSecondaryResourceTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_tasksecondaryresource',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.tasksecondaryresource',),
        ),
        migrations.CreateModel(
            name='TaskTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_task',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.task',),
        ),
        migrations.CreateModel(
            name='TaskTypeLinkTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_tasktypelink',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.tasktypelink',),
        ),
        migrations.CreateModel(
            name='TicketCategoryTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_ticketcategory',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.ticketcategory',),
        ),
        migrations.CreateModel(
            name='TicketNoteTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_ticketnote',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.ticketnote',),
        ),
        migrations.CreateModel(
            name='TicketSecondaryResourceTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_ticketsecondaryresource',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.ticketsecondaryresource',),
        ),
        migrations.CreateModel(
            name='TicketTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_ticket',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.ticket',),
        ),
        migrations.CreateModel(
            name='TicketTypeTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_tickettype',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.tickettype',),
        ),
        migrations.CreateModel(
            name='TimeEntryTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_timeentry',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.timeentry',),
        ),
        migrations.CreateModel(
            name='UseTypeTracker',
            fields=[
            ],
            options={
                'db_table': 'djautotask_usetype',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djautotask.usetype',),
        ),
    ]



# Generated by Django 2.1.11 on 2019-09-23 11:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0008_auto_20190917_1454'),
        ('djautotask', '0009_auto_20190920_1648'),
    ]

    operations = [
    ]



# Generated by Django 2.1.11 on 2020-01-27 11:57

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0035_timeentry'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='displaycolor',
            name='parent_value',
        ),
        migrations.RemoveField(
            model_name='issuetype',
            name='parent_value',
        ),
        migrations.RemoveField(
            model_name='licensetype',
            name='parent_value',
        ),
        migrations.RemoveField(
            model_name='priority',
            name='parent_value',
        ),
        migrations.RemoveField(
            model_name='projectstatus',
            name='parent_value',
        ),
        migrations.RemoveField(
            model_name='projecttype',
            name='parent_value',
        ),
        migrations.RemoveField(
            model_name='queue',
            name='parent_value',
        ),
        migrations.RemoveField(
            model_name='source',
            name='parent_value',
        ),
        migrations.RemoveField(
            model_name='status',
            name='parent_value',
        ),
        migrations.RemoveField(
            model_name='subissuetype',
            name='parent_value',
        ),
        migrations.RemoveField(
            model_name='tickettype',
            name='parent_value',
        ),
    ]



# Generated by Django 2.1.11 on 2019-10-29 16:21

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0017_auto_20191021_1028'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ticket',
            name='category',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djautotask.TicketCategory'),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='issue_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djautotask.IssueType'),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='project',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djautotask.Project'),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='source',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djautotask.Source'),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='sub_issue_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djautotask.SubIssueType'),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djautotask.TicketType'),
        ),
    ]



# Generated by Django 4.0.7 on 2022-10-11 13:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0106_ticketnote_created_by_contact'),
    ]

    operations = [
        migrations.AddField(
            model_name='contact',
            name='extension',
            field=models.CharField(blank=True, max_length=10, null=True),
        ),
    ]



# Generated by Django 2.1.11 on 2019-10-30 09:05

from django.db import migrations
import django.db.models.manager


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0018_auto_20191029_1634'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='resource',
            managers=[
                ('regular_objects', django.db.models.manager.Manager()),
            ],
        ),
    ]



# Generated by Django 3.1.2 on 2020-10-22 18:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0084_projectudftracker_taskudftracker'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='udf',
            field=models.JSONField(blank=True, default=dict, null=True),
        ),
        migrations.AddField(
            model_name='task',
            name='udf',
            field=models.JSONField(blank=True, default=dict, null=True),
        ),
    ]



# Generated by Django 2.1.14 on 2020-05-21 16:15

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0067_auto_20200519_1100'),
        ('djautotask', '0067_merge_20200514_1713'),
    ]

    operations = [
    ]



# Generated by Django 2.1.14 on 2019-11-25 11:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0026_auto_20191125_0952'),
    ]

    operations = [
        migrations.RenameField(
            model_name='task',
            old_name='priority_label',
            new_name='priority',
        ),
    ]
